#####
###  Abracadabra Object - pre-configured objects with only one click
##   Copyright (C) 2003 Maik Jablonski (maik.jablonski@uni-bielefeld.de)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

from re import sub
from urllib import quote
from DateTime import DateTime
from Globals import DTMLFile, InitializeClass
from CharacterMapping import CharacterMapping
            
def manage_addAbracadabraObject(self, id, object_id, title="", auto_id=None, REQUEST=None):
    """ Clone an object from standard_object_folder
        and execute a corresponding script if exists.
    """
    
    # Check for place-holders from form-submit
    # Please note: ID & TITLE can't be used
    # as ids for Zope-objects anyway.
    if id in ["ID","[AUTO-ID]"]: id=""
    if title=="TITLE": title=""

    # Get the famous standard_object_folder
    stdof = self.standard_object_folder.aq_explicit
    
    # check for auto_id in the standard_object_folder    
    if not auto_id:
        auto_id = stdof.getProperty('auto_id', None)

    # Calculate an automatic id if no id is given
    if object_id and auto_id and not id:
        # we have a list? search the entry for object_id
        if type(auto_id) == type([]):
            for line in auto_id:
                if ':' in line:
                    (objStr,fmtStr) = line.split(':', 1)
                    if objStr == object_id:
                        auto_id = fmtStr
                        break
                
        # auto_id as string-property
        if type(auto_id) == type(''):           
            if title and auto_id.find('%s')<>-1:
                # Cook id out of title
                id = auto_id % title
            else:
                tmpId = len(self.objectIds())               
                # to be sure: test and search for a free id...
                while hasattr(self, auto_id % tmpId): 
                    tmpId = tmpId + 1
                id = auto_id % tmpId

        # auto_id as integer? Count it up...
        elif type(auto_id) == type(0):
            id = auto_id
            while hasattr(self, str(id)):
                id = id + 1
            stdof.manage_changeProperties({'auto_id': id+1})

        # Fallback is timestamp as id
        elif auto_id:
            id = DateTime().millis()
            # to be sure: test and search for a free id...
            while hasattr(self, "%s" % str(id)):
                id = id + 1 

    if id and object_id:
        id = str(id)
        # we need a valid id - so we should take care...
        # replace chars from the CharacterMap
        for fromChar in CharacterMapping.keys():
            id = id.replace(fromChar, CharacterMapping[fromChar])

        # replace all chars which are not letters or digits with underscore
        id = sub('^_*|_*$', '', sub('[^\w.-]+','_', id))
    
        obj = self.manage_clone(getattr(stdof, object_id), id, REQUEST)
        obj.title = str(title)

        # Call corresponding PythonScript with object_id_
        if hasattr(stdof, object_id+'_'):
            callObj = getattr(stdof, object_id+'_')

            # Python-Script? Call it with the correct bindings
            if callObj.meta_type == 'Script (Python)':                 
                callObj._exec({'context': obj,
                               'container': callObj.aq_parent,
                               'script': callObj,
                               'traverse_subpath': ''},[], {})

                # Maybe the script renamed the object?
                if id <> obj.getId():
                    id = obj.getId()
                
    if REQUEST:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
                
        if hasattr(self, id):
            u = "%s/%s/manage_workspace" % (u, quote(id))
        else:
            error = "manage_tabs_message=ERROR:+Valid+id+is+required"
            u = "%s/manage_main?%s" % (u, error)
               
        REQUEST.RESPONSE.redirect(u)

    if hasattr(self, id):
        return id
        
manage_addAbracadabraObjectForm = DTMLFile('dtml/addAbracadabraObject',
                                                                  globals())

class AbracadabraObject:
    """ AbracadabraObject - pre-configured objects with only one click """

    meta_type = "Abracadabra Object"
    __ac_permissions__ = ((('Abracadabra Object: Show StandardSelectBox'),()),)
    
InitializeClass(AbracadabraObject)
